﻿using Connected.Utilities;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;

namespace Connected.ComponentsN;
public partial class Select<ValueType>
{

	[Parameter]
	public bool SearchFieldEnabled { get; set; } = false;

	[Parameter]
	public int MaxVisibleItems { get; set; } = 5;

	[Parameter]
	public List<SelectItem<ValueType>> Items
	{
		set {
			_originalList = value;
			_itemsWithFilter= value;
			_itemsToShow = _itemsWithFilter.Skip(Offset).Take(MaxVisibleItems).ToList();
		}
	}

	private void SetPage(int page)
	{
		_selectedPage = page;
		SetLists();
		StateHasChanged();
	}

	private void SetLists()
	{
		if (FilterText.Length > 0)
		{
			_itemsWithFilter = _originalList.Where(i => i.Value.ToString().Contains(FilterText) || i.Text.ToString().Contains(FilterText)).ToList();
		}
		else
		{
			_itemsWithFilter = _originalList;
		}
		_itemsToShow = _itemsWithFilter.Skip(Offset).Take(MaxVisibleItems).ToList();
	}

	private int Offset
	{
		get
		{
			return (_selectedPage - 1) * MaxVisibleItems;
		}
	}
	private List<SelectItem<ValueType>> _itemsWithFilter;
		private List<SelectItem<ValueType>> _itemsToShow;
	private List<SelectItem<ValueType>> _originalList;

	[Parameter]
	public EventCallback<string> ValueChanged { get; set; }
	private async Task Search(ChangeEventArgs args)
	{
		FilterText = args.Value.ToString();
		SetLists();
		_pagination();
		await ValueChanged.InvokeAsync(args.Value.ToString());
	}

	private int ItemsCount => _itemsWithFilter.Count();

	private int Height = 20;

	private bool SearchFieldVisibility = false;

	private void HeightOnFull()
	{
		if (Height < 400)
		{
			SearchFieldVisibility = true;
			Height = 400;
			StateHasChanged();
		}
	}

	private void HeightToDefault()
	{
		if (Height > 20)
		{
			SearchFieldVisibility = false;
			Height = 20;
			StateHasChanged();
		}
	}

	private string GetHeight()
	{
		return Height.ToString() + "px;";
	}


	[Parameter]
	public string FilterText { get; set; } = string.Empty;

	private int numberOfPages = 1;
	private void _pagination()
	{
		int result = 1;

		int totalItems = _itemsWithFilter.Count;
		if (MaxVisibleItems < totalItems)
		{
			result = (int)(totalItems / MaxVisibleItems);
			if (totalItems % MaxVisibleItems > 0) result++;
		}
		if (!numberOfPages.Equals(result))
		{
			numberOfPages = result;
			StateHasChanged();
		}
		
	}
	private int _selectedPage = 1;

	[Parameter]
	public string ClassList { get; set; } = string.Empty;
	/// <summary>
	/// Contains the default container classlist and the user defined classes.
	/// </summary>
	private string CompiledClassList
	{
		get
		{
			return new CssBuilder()
				.AddClass(ClassList)
				.Build();
		}
	}

	protected override Task OnParametersSetAsync()
	{
		_pagination();
		SetLists();
		return base.OnParametersSetAsync();
	}
}
